#!/usr/bin/env perl6

use Data::Reshapers;
use Pretty::Table;
use Hash::Merge;

my @tbl = get-titanic-dataset(headers => 'auto');

#say @tbl.roll(6);
say to-pretty-table(@tbl.roll(6));

# select columns
my @tbl2 = @tbl.map({ %($_.grep({ $_.key (elem) ["id", "passengerSex", "passengerClass"] })) });
say to-pretty-table(@tbl2.roll(6));

my @tbl3 = @tbl.map({ %($_.grep({ $_.key (elem) ["id", "passengerAge", "passengerSurvival"] })) });
say to-pretty-table(@tbl3.roll(6));

# Filter
#say to-pretty-table(@tbl.grep({ 10 <= $_{"passengerAge"}.Int <= 20 }).roll(12));

# Group by and count
#say @tbl.classify({ $_{"passengerSurvival"}}).map({ $_.key => $_.value.elems });

#say @tbl.classify({ $_{"passengerClass", "passengerSex"}.join(' ◊ ') }).map({ $_.key => $_.value.elems });

# Group by
#say @tbl.classify({ $_{"passengerClass"}}, :as{ $_.elems} );

# Inner join

# From https://titanwolf.org/Network/Articles/Article?AID=8dd25a53-f1d5-4cc9-a918-5fa891777552
sub hash-join(@a, &a, @b, &b) {
    my %hash := @b.classify(&b);
    @a.map: -> $a {
        |(%hash{&a($a)} // next).map: -> $b { merge-hash($a, $b) }
    }
}

my @tblRes = hash-join @tbl2, *{'id'}, @tbl3, *{'id'};

say @tblRes.roll(5);
say to-pretty-table(@tblRes.roll(5));

## Rename column names
## Re-programmed to Raku : https://metacpan.org/pod/Hash::Rename#hash_rename
sub hash-rename(%hash, *%args) {
    my %new_hash;
    for %hash.keys.sort -> $key {
        my $new_key = $key;

        if %args{$key}:exists {
            $new_key = %args{$key}
        } elsif %args<code>:exists {
            $new_key = %args<code>($key)
        } else {
            $new_key = (%args<prepend> ~ $key) if defined %args<prepend>;
            $new_key = ($new_key ~ %args<append>) if defined %args<append>;
        }

        if %args<strict>:exists and %args<strict> and %new_hash{$new_key}:exists {
            die "duplicate result key [$new_key] from original key [$key]\n";
        }

        # apply 'recurse' option, if given
        my $val = %hash{$key};
        if %args<recurse>:exists and %args<recurse> { 
            hash-rename($val, |%args);
        }

        %new_hash{$new_key} = $val;
    }
    %new_hash
}

say @tbl[12];
say hash-rename(@tbl[12], passengerAge => 'age', passengerClass => 'class');
say hash-rename(@tbl[12], append => '-NEW', prepend => 'SEE-');
say hash-rename(@tbl[12], append => '-NEW', prepend => 'SEE-', passengerAge => 'age');

